<?php

namespace Drupal\Tests\node_revisions_autoclean\Functional;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\node\Entity\Node;
use Drupal\node_revisions_autoclean\Services\RevisionsManager;
use Drupal\Tests\BrowserTestBase;

/**
 * Class NodeRevisionsAutocleanTest
 * @package Drupal\Tests\node_revisions_autoclean\Functional
 * @group node_revisions_autoclean
 * @ingroup node_revisions_autoclean
 */
class NodeRevisionsAutocleanTest extends BrowserTestBase
{
    use StringTranslationTrait;

    public static $modules = ['node', 'node_revisions_autoclean'];

    protected function setUp()
    {
        parent::setUp();
        //creates default node types
        $config = \Drupal::configFactory()->getEditable('node_revisions_autoclean.settings');
        $config
            ->set('enable_on_cron', 0)
            ->set('enable_on_node_update', 0)
            ->save(true);
        $type = $this->container->get('entity_type.manager')->getStorage('node_type')
            ->create([
                'type' => 'article',
                'name' => 'Article',
            ]);
        $type->save();
        $type = $this->container->get('entity_type.manager')->getStorage('node_type')
            ->create([
                'type' => 'page',
                'name' => 'Page',
            ]);
        $type->save();
        $this->container->get('router.builder')->rebuild();
    }


    private function initializeSettings($enabledAutoclean = false)
    {
        $config = \Drupal::configFactory()->getEditable('node_revisions_autoclean.settings');
        $config
            ->set('enable_on_cron', $enabledAutoclean ? 1 : 0)
            ->set('enable_on_node_update', $enabledAutoclean ? 1 : 0)
            ->set('node.page', $enabledAutoclean ? 2 : 0)
            ->set('node.article', $enabledAutoclean ? 2 : 0)
            ->save(true);
    }

    /**
     * Build
     * @param string $title
     * @param string $type
     * @return Node $node
     */
    public function createNodeWithRevisions($title, $type = 'page')
    {
        $user = $this->drupalCreateUser();
        $node = Node::create([
            'type' => $type,
            'title' => '1'
        ]);
        $node->setPublished(true);
        $node->save();
        //draft
        $this->addRevision($node, $user, false);
        $this->addRevision($node, $user, true);
        $this->addRevision($node, $user, false);
        $this->addRevision($node, $user, true);
        $this->addRevision($node, $user, false);
        $this->addRevision($node, $user, true);
        $this->addRevision($node, $user, true);
        $this->addRevision($node, $user, false);
        $this->addRevision($node, $user, false);
        $this->addRevision($node, $user, false);
        $this->addRevision($node, $user, false);
        return $node;
    }

    public function addRevision(Node &$node, $user, $published = false)
    {
        $node->setNewRevision(true);
        $node->setRevisionCreationTime(time());
        $node->setRevisionUserId($user->id());
        $node->setPublished($published);
        $node->save();
    }

    //protected function getConfigSchemaExclusions() {
    //return array_merge(parent::getConfigSchemaExclusions(), ['node_revisions_autoclean.settings']);
    //}

    /**
     * Tests permissions anonymous
     */
    public function testAdminConfigAnonymousAccess()
    {
        $assert = $this->assertSession();

        $user = $this->drupalCreateUser();
        $this->drupalLogin($user);
        $this->drupalGet('/admin/config/content/revisions-autoclean');
        $assert->statusCodeEquals(403);

    }

    /**
     * Test permissions admin
     */
    public function testAdminConfigAdminAccess()
    {
        $assert = $this->assertSession();
        $user = $this->drupalCreateUser(['configure revisions autoclean settings']);
        $this->drupalLogin($user);
        $this->drupalGet('/admin/config/content/revisions-autoclean');
        $assert->statusCodeEquals(200);
    }

    /**
     * Test initial configuration
     */
    public function testModuleInitialConfiguration()
    {
        $config = \Drupal::configFactory()->get('node_revisions_autoclean.settings');
        $this->assertNotNull($config->get('enable_on_cron'), $this->t("Configuration enable_on_cron missing in node_revisions_autoclean.settings"));
        $this->assertNotNull($config->get('enable_on_node_update'), $this->t("Configuration enable_on_node_update missing in node_revisions_autoclean.settings"));
        $this->assertEquals($config->get('enable_on_cron'), 0);
        $this->assertEquals($config->get('enable_on_node_update'), 0);
    }

    /**
     * Test initial config form with article & page
     */
    public function testConfigForm()
    {
        $config = \Drupal::configFactory()->get('node_revisions_autoclean.settings');
        $assert = $this->assertSession();

        $user = $this->drupalCreateUser(['configure revisions autoclean settings']);
        $this->drupalLogin($user);
        $this->drupalGet('/admin/config/content/revisions-autoclean');

        $assert->checkboxNotChecked('enable_on_cron');
        $assert->checkboxNotChecked('enable_on_node_update');

        $assert->fieldExists('node__article');
        $assert->fieldValueEquals('node__article', 0);

        $assert->fieldExists('node__page');
        $assert->fieldValueEquals('node__page', 0);
    }

    /**
     * Test post form
     */
    public function testDrupalPostForm()
    {
        $assert = $this->assertSession();
        $user = $this->drupalCreateUser(['configure revisions autoclean settings']);
        $this->drupalLogin($user);
        $this->drupalGet('/admin/config/content/revisions-autoclean');

        $this->drupalPostForm(null, [
            'enable_on_cron' => '0',
            'enable_on_node_update' => '1',
            'node__article' => '2',
            'node__page' => '2',
        ], $this->t('Submit'));

        $config = \Drupal::config('node_revisions_autoclean.settings');

        $this->assertEquals('0', $config->get('enable_on_cron'), 'Value for enable_on_cron ' . $config->get('enable_on_cron'));
        $this->assertEquals('1', $config->get('enable_on_node_update'), 'Value for enable_on_node_update : ' . $config->get('enable_on_node_update'));
        $this->assertEquals('2', $config->get('node.article'), 'Value for node.article : ' . $config->get('node.article'));
        $this->assertEquals('2', $config->get('node.page'), 'Value for node.page : ' . $config->get('node.page'));

        $assert->pageTextContains('Node revisions settings have been updated.');
    }

    /**
     * Test new configuration
     */
    public function testConfiguration()
    {
        $assert = $this->assertSession();
        $this->initializeSettings(true);
        $config = \Drupal::config('node_revisions_autoclean.settings');
        $this->assertEquals(1, $config->get('enable_on_cron'), 'Value for enable_on_cron ' . $config->get('enable_on_cron'));
        $this->assertEquals(1, $config->get('enable_on_node_update'), 'Value for enable_on_node_update : ' . $config->get('enable_on_node_update'));
        $this->assertEquals(2, $config->get('node.article'), 'Value for node.article : ' . $config->get('node.article'));
        $this->assertEquals(2, $config->get('node.page'), 'Value for node.page : ' . $config->get('node.page'));
    }

    /**
     * test revisions
     */
    public function testInitialRevisions()
    {
        $this->initializeSettings(false);
        $node = $this->createNodeWithRevisions('1', 'page');
        /* @var $revisionsManager RevisionsManager */
        $revisionsManager = \Drupal::service('node_revisions_autoclean.revisions_manager');
        $revisions = $revisionsManager->loadRevisions($node);

        $this->assertEquals(12, count($revisions), 'Revisions initial : ' . count($revisions));
    }

    public function testDeleteRevisions()
    {
        $node = $this->createNodeWithRevisions('1', 'page');
        /* @var $revisionsManager RevisionsManager */
        $revisionsManager = \Drupal::service('node_revisions_autoclean.revisions_manager');
        $revisions = $revisionsManager->loadRevisions($node);
        //createNodeWithRevisions creates 12 revisions, check :
        $this->assertEquals(12, count($revisions), 'Revisions initial : ' . count($revisions));

        $this->initializeSettings(true);
        $user = $this->drupalCreateUser(['configure revisions autoclean settings']);
        $this->addRevision($node, $user, true);
        $revisionsAfterSave = $revisionsManager->loadRevisions($node);
        //There should be 3 revisions
        $this->assertEquals(3, count($revisionsAfterSave), 'Revisions after save : ' . count($revisionsAfterSave));
        $this->addRevision($node, $user, false);
        $this->addRevision($node, $user, false);
        $this->addRevision($node, $user, false);
        $revisionsAfterSave2 = $revisionsManager->loadRevisions($node);
        // There should be 6 revisions : all last drafts + 3
        $this->assertEquals(6, count($revisionsAfterSave2), 'Revisions after insert & save  : ' . count($revisionsAfterSave2));
        $this->addRevision($node, $user, true);
        $this->addRevision($node, $user, true);
        $revisionsAfterSave3 = $revisionsManager->loadRevisions($node);

        //there should be 3 (current + 2 in settings)
        $this->assertEquals(3, count($revisionsAfterSave3), 'Revisions after insert & save  : ' . count($revisionsAfterSave3));

        //disable autoclean
        $this->initializeSettings(false);
        $this->addRevision($node, $user, true);
        $revisionsAfterSave4 = $revisionsManager->loadRevisions($node);
        //there should be 4
        $this->assertEquals(4, count($revisionsAfterSave4), 'Revisions after insert & save  : ' . count($revisionsAfterSave4));
    }

}